﻿using System;
using System.Collections.Generic;
using System.Text;
using RGiesecke.DllExport;
using System.Runtime.InteropServices;

namespace TradePumpNet
{
    /// <summary>
    /// External Exports like a C++ dll
    /// Written by: Rob Hull trading software services rob@hullhaven.com
    /// I removed my actual implementation for this datapump example
    /// To implement add code to the datapump class for GetBars and OnTick
    /// </summary>
    internal static class UnmanagedExports
    {
        static Datapump _pump = null;

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern int NSTSETUPDATA(string ticker, string description, float barInterval,
            int numColumns, string category, double firstDateAvailable);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern int NSTSETLABEL(string ticker, float barInterval, int col, string label);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern int NSTSETVALUE(string ticker, float barInterval, int col, double val);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern int NSTCOMMITROW(string ticker, float barInterval);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern void SetDataFeedhwnd(int hWnd);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern void UpdatePercent(int Percent);

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern void RequireUserIDPassword();

        [DllImport("NSTFeed.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern void GetUserIDPassword(ref string UserID, ref string Password);

        // EXPORTS: This is where the magic is for exporting a .net method as unmanaged. The credit goes to
        // http://sites.google.com/site/robertgiesecke/Home/uploads/unmanagedexports 

        [DllExport("UserIDPasswordChanged", CallingConvention = System.Runtime.InteropServices.CallingConvention.StdCall)]
        static int UserIDPasswordChanged(string user, string pass)
        {
            Logger.Log("UserIDPasswordChanged", System.Diagnostics.EventLogEntryType.Information);
            return 1;
        }

        [DllExport("RemoveTicker", CallingConvention = System.Runtime.InteropServices.CallingConvention.StdCall)]
        static int RemoveTicker(string ticker, float barInterval)
        {
            Logger.Log("RemoveTicker", System.Diagnostics.EventLogEntryType.Information);
            try
            {
                Startup();
                float barInt = (float)((int)barInterval * -1); 
                Logger.Log("Request Ticker:", System.Diagnostics.EventLogEntryType.Information);
                _pump.RemoveSymbol(ticker, (int)barInt);
                return 1;
            }
            catch (Exception ex)
            {
                Logger.Log(ex);
                return 0;
            }
        }

        [DllExport("RequestTicker", CallingConvention = System.Runtime.InteropServices.CallingConvention.StdCall)]
        static int RequestTicker(string ticker, string description, string category, double startdate, float BarInterval, string ColumnNames)
        {
            Logger.Log("RequestTicker:" + ticker + "StartDate:" + startdate.ToString() + " Bar Interval:" + BarInterval + " ColumnNames:" + ColumnNames
                    , System.Diagnostics.EventLogEntryType.Information);
            try
            {
                Startup();
                int tickColumns = 3; //tick data
                float barInt =  (float)((int)BarInterval * -1); 
                Logger.Log("Request Ticker:", System.Diagnostics.EventLogEntryType.Information);
                Logger.Log(ticker, System.Diagnostics.EventLogEntryType.Information);

                if (NSTSETUPDATA(ticker, description, barInt, tickColumns, category, startdate) == 0)
                {
                    Logger.Log("Error Requiest Ticker:NSTSETUPDATA", System.Diagnostics.EventLogEntryType.Error);
                    return 0;
                }
                if (NSTSETLABEL(ticker, barInt, 1, @"Date/Time") == 0)
                {
                    Logger.Log("Error Requiest Ticker:NSTSETLABEL", System.Diagnostics.EventLogEntryType.Error);
                    return 0;
                }
                if (NSTSETLABEL(ticker, barInt, 2, "Close") == 0)
                {
                    Logger.Log("Error Requiest Ticker:NSTSETLABEL", System.Diagnostics.EventLogEntryType.Error);
                    return 0;
                } 
                if (NSTSETLABEL(ticker, barInt, 3, "Volume") == 0)
                {
                    Logger.Log("Error Requiest Ticker:NSTSETLABEL", System.Diagnostics.EventLogEntryType.Error);
                    return 0;
                }
                _pump.GetBars(ticker, startdate, (int)BarInterval);

                UpdatePercent(100);
                Logger.Log("Finished getting tick:", System.Diagnostics.EventLogEntryType.Information);

                _pump.AddSymbol(ticker, (int)barInt);

                return 1;
            }
            catch (Exception ex)
            {
                Logger.Log(ex);
                return 0;
            }
        }
        
        static void Startup()
        {
            if (_pump == null)
            {
                _pump = new Datapump();
                _pump.Start();
            }
        }

    }
}
